//
//  LoginViewModel.swift
//  iOSPOCs
//
//  Created by Roman Sorochak on 23.03.2022.
//

import SwiftUI
import FirebaseAuth
import AuthenticationServices
import CryptoKit

// MARK - LoginViewModel

class LoginViewModel: NSObject, ObservableObject, Resultable, Alertable {
    
    
    // MARK: Result

    enum Result: Equatable {
        case userSignedIn
        case emailIsNotVerified
        case phoneNumberIsRequiredToSignIn(resolver: MultiFactorResolver, hint: PhoneMultiFactorInfo)
    }

    var onResult: ((Result) -> Void)?

    // MAKR: Properties

    @Published var email: String = Environment.testEmail
    @Published var password: String = Environment.testPassword
    @Published var isLoading: Bool = false
    @Published var isSignInWithAppleLoading: Bool = false
    var error: Error? {
        didSet {
            isErrorAlertShowing = error != nil
        }
    }
    @Published var isErrorAlertShowing: Bool = false
    // Unhashed nonce.
    private var currentNonce: String?

    // MARK: Actions

    func loginAction() {
        login()
    }

    func signInWithAppleAction() {
        startSignInWithAppleFlow()
    }

    // MARK: Private

    private func login() {
        isLoading = true

        Auth.auth().signIn(withEmail: email, password: password) { [weak self] authDataResult, error in
            guard let self = self else { return }

            self.isLoading = false

            if let error = error {
                print(error)

                let authError = error as NSError
                if authError.code == AuthErrorCode.secondFactorRequired.rawValue,
                   let resolver = authError.userInfo[AuthErrorUserInfoMultiFactorResolverKey] as? MultiFactorResolver,
                   let hint = resolver.hints.first as? PhoneMultiFactorInfo {
                    self.onResult?(.phoneNumberIsRequiredToSignIn(resolver: resolver, hint: hint))
                } else {
                    self.error = error
                }
                return
            }

            guard authDataResult?.user.isEmailVerified == true else {
                self.onResult?(.emailIsNotVerified)
                return
            }
            
            self.onResult?(.userSignedIn)
        }
    }

    func startSignInWithAppleFlow() {
        let nonce = randomNonceString()
        currentNonce = nonce
        let appleIDProvider = ASAuthorizationAppleIDProvider()
        let request = appleIDProvider.createRequest()
        request.requestedScopes = [.fullName, .email]
        request.nonce = sha256(nonce)

        let authorizationController = ASAuthorizationController(authorizationRequests: [request])
        authorizationController.delegate = self
        authorizationController.performRequests()
    }
}

extension LoginViewModel: ASAuthorizationControllerDelegate {

    func authorizationController(controller: ASAuthorizationController,
                                 didCompleteWithAuthorization authorization: ASAuthorization) {
        guard let appleIDCredential = authorization.credential as? ASAuthorizationAppleIDCredential else {
            return
        }
        guard let nonce = currentNonce else {
            fatalError("Invalid state: A login callback was received, but no login request was sent.")
        }
        guard let appleIDToken = appleIDCredential.identityToken else {
            print("Unable to fetch identity token")
            return
        }
        guard let idTokenString = String(data: appleIDToken, encoding: .utf8) else {
            print("Unable to serialize token string from data: \(appleIDToken.debugDescription)")
            return
        }
        print("\n ** ASAuthorizationAppleIDCredential - \(#function)** \n")
        print(appleIDCredential.email ?? "Email not available.")
        print(appleIDCredential.fullName ?? "fullname not available")
        print(appleIDCredential.fullName?.givenName ?? "givenName not available")
        print(appleIDCredential.fullName?.middleName ?? "middleName not available")
        print(appleIDCredential.fullName?.familyName ?? "Familyname not available")
        print(appleIDCredential.user)  // This is a user identifier
        print(appleIDCredential.identityToken?.base64EncodedString() ?? "Identity token not available") //JWT Token
        print(appleIDCredential.authorizationCode?.base64EncodedString() ?? "Authorization code not available")

        // Initialize a Firebase credential.
        let credential = OAuthProvider.credential(withProviderID: "apple.com",
                                                  idToken: idTokenString,
                                                  rawNonce: nonce)
        // Sign in with Firebase.
        Auth.auth().signIn(with: credential) { (authResult, error) in
            if let error = error {
                print(error)

                let authError = error as NSError
                if authError.code == AuthErrorCode.secondFactorRequired.rawValue,
                   let resolver = authError.userInfo[AuthErrorUserInfoMultiFactorResolverKey] as? MultiFactorResolver,
                   let hint = resolver.hints.first as? PhoneMultiFactorInfo {
                    self.onResult?(.phoneNumberIsRequiredToSignIn(resolver: resolver, hint: hint))
                } else {
                    self.error = error
                }
                return
            }
            // User is signed in to Firebase with Apple.
            // ...
            self.onResult?(.userSignedIn)
        }
    }

    func authorizationController(controller: ASAuthorizationController,
                                 didCompleteWithError error: Error) {
        print("\n -- ASAuthorizationControllerDelegate -\(#function) -- \n")
        print(error)
    }
}

private extension LoginViewModel {

    // Adapted from https://auth0.com/docs/api-auth/tutorials/nonce#generate-a-cryptographically-random-nonce
    func randomNonceString(length: Int = 32) -> String {
        precondition(length > 0)
        let charset: [Character] =
        Array("0123456789ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrstuvwxyz-._")
        var result = ""
        var remainingLength = length

        while remainingLength > 0 {
            let randoms: [UInt8] = (0 ..< 16).map { _ in
                var random: UInt8 = 0
                let errorCode = SecRandomCopyBytes(kSecRandomDefault, 1, &random)
                if errorCode != errSecSuccess {
                    fatalError(
                        "Unable to generate nonce. SecRandomCopyBytes failed with OSStatus \(errorCode)"
                    )
                }
                return random
            }

            randoms.forEach { random in
                if remainingLength == 0 {
                    return
                }

                if random < charset.count {
                    result.append(charset[Int(random)])
                    remainingLength -= 1
                }
            }
        }

        return result
    }

    @available(iOS 13, *)
    private func sha256(_ input: String) -> String {
        let inputData = Data(input.utf8)
        let hashedData = SHA256.hash(data: inputData)
        let hashString = hashedData.compactMap {
            String(format: "%02x", $0)
        }.joined()

        return hashString
    }
}
