//
//  LoginCoordinator.swift
//  iOSPOCs
//
//  Created by Roman Sorochak on 24.03.2022.
//

import SwiftUI
import FirebaseAuth

// MARK: - LoginCoordinator

class LoginCoordinator: Coordinator, Resultable {

    // MARK: Result

    enum Result: Equatable {
        case userSignedIn
        case userSignedInEmailVerified
        case userSignedInEmailAndPhoneVerified
    }

    var onResult: ((Result) -> Void)?

    // MARK: Route

    enum Route {
        case verifyEmail(coordinator: VerifyEmailCoordinator)
        case phoneNumber(coordinator: PhoneNumberCoordinator)
    }

    @Published var route: Route?

    // MARK: Properties

    let loginViewModel: LoginViewModel

    // MARK: Life cycle

    init() {
        loginViewModel = LoginViewModel()
        loginViewModel.onResult = { [weak self] result in
            switch result {
            case .userSignedIn:
                self?.onResult?(.userSignedIn)
            case .emailIsNotVerified:
                break
            case let .phoneNumberIsRequiredToSignIn(resolver, hint):
                self?.routePhoneNumberView(resolver: resolver, hint: hint)
            }
        }
    }

    // MARK: Routes

    private func routeVerifyEmail() {
        let coordinator = VerifyEmailCoordinator()
        coordinator.onResult = { [weak self] result in
            switch result {
            case .emailVerified:
                self?.route = nil
                self?.onResult?(.userSignedInEmailVerified)
            case .emailAndPhoneVerified:
                self?.route = nil
                self?.onResult?(.userSignedInEmailAndPhoneVerified)
            }
        }
        route = .verifyEmail(coordinator: coordinator)
    }

    private func routePhoneNumberView(resolver: MultiFactorResolver?, hint: PhoneMultiFactorInfo?) {
        let coordinator = PhoneNumberCoordinator(resolver: resolver, hint: hint)
        coordinator.onResult = { [weak self] result in
            switch result {
            case .phoneNumberVerified:
                self?.route = nil
                self?.onResult?(.userSignedIn)
            case .phoneNumberVerificationSkipped:
                self?.route = nil
                self?.onResult?(.userSignedIn)
            }
        }
        route = .phoneNumber(coordinator: coordinator)
    }
}

