//
//  OTPViewModel.swift
//  iOSPOCs
//
//  Created by Roman Sorochak on 21.03.2022.
//

import SwiftUI
import FirebaseAuth

// MARK: - OTPViewModel

class OTPViewModel: ObservableObject, Resultable, Alertable {

    // MARK: Result

    enum Result: Equatable {
        case otpVerified
    }

    var onResult: ((Result) -> Void)?

    // MARK: Properties

    let verificationId: String
    let phoneNumber: String
    let resolver: MultiFactorResolver?

    @Published var otpCode: String = Environment.testOTP
    @Published var isLoading: Bool = false
    var error: Error? {
        didSet {
            isErrorAlertShowing = error != nil
        }
    }
    @Published var isErrorAlertShowing: Bool = false

    init(verificationId: String,
         phoneNumber: String,
         resolver: MultiFactorResolver?) {
        self.verificationId = verificationId
        self.phoneNumber = phoneNumber
        self.resolver = resolver
    }

    // MARK: Actions

    func continueAction() {
        resolveOTP()
    }

    // MARK: Private

    private func resolveOTP() {
        isLoading = true

        let credential = PhoneAuthProvider.provider().credential(
            withVerificationID: verificationId,
            verificationCode: otpCode
        )

        let assertion = PhoneMultiFactorGenerator.assertion(with: credential)

        if let resolver = resolver {
            resolveSignIn(assertion: assertion, resolver: resolver)
        } else {
            enroll2Factor(assertion: assertion)
        }
    }

    private func enroll2Factor(assertion: MultiFactorAssertion) {
        Auth.auth().currentUser?.multiFactor.enroll(
            with: assertion,
            displayName: phoneNumber,
            completion: { [weak self] error in
                guard let self = self else { return }

                self.isLoading = false

                if let error = error {
                    print(error)
                    self.error = error
                    return
                }

                Auth.auth().currentUser?.reload(completion: { [weak self] error in
                    guard let self = self else { return }

                    if let error = error {
                        print(error)
                        self.error = error
                        return
                    }

                    self.onResult?(.otpVerified)
                })
            })
    }

    private func resolveSignIn(assertion: MultiFactorAssertion, resolver: MultiFactorResolver) {
        resolver.resolveSignIn(with: assertion, completion: { [weak self] authDataResult, error in
            guard let self = self else { return }

            self.isLoading = false

            if let error = error {
                print(error)
                self.error = error
                return
            }

            guard let _ = authDataResult?.user else {
                //TODO: handle enexpected case
                return
            }

            self.onResult?(.otpVerified)
        })
    }
}

