//
//  PhoneNumberViewModel.swift
//  iOSPOCs
//
//  Created by Roman Sorochak on 21.03.2022.
//

import SwiftUI
import FirebaseAuth

// MARK: - PhoneNumberViewModel

class PhoneNumberViewModel: ObservableObject, Resultable, Alertable {

    // MARK: Result

    enum Result: Equatable {
        case skipped
        case optCodeDidSent(verificationId: String, phoneNumber: String, resolver: MultiFactorResolver?)
        case requireRecentLogin
    }

    var onResult: ((Result) -> Void)?

    // MARK: Properties

    private let resolver: MultiFactorResolver?
    private let hint: PhoneMultiFactorInfo?

    let isAbleToSkip: Bool
    @Published var phoneNumberText: String = Environment.testPhoneNumber
    @Published var isLoading: Bool = false
    var error: Error? {
        didSet {
            isErrorAlertShowing = error != nil
        }
    }
    @Published var isErrorAlertShowing: Bool = false

    // MARK: Life cycle

    init(resolver: MultiFactorResolver?, hint: PhoneMultiFactorInfo?) {
        self.resolver = resolver
        self.hint = hint
        self.isAbleToSkip = resolver == nil
    }

    // MARK: Actions

    func continueAction() {
        verifyPhoneNumber()
    }

    func skipAction() {
        onResult?(.skipped)
    }

    func okAlertAction() {
        if let authError = error as NSError?,
           authError.code == AuthErrorCode.requiresRecentLogin.rawValue {
            onResult?(.requireRecentLogin)
        }
    }

    // MARK: Private

    private func verifyPhoneNumber() {
        isLoading = true

        if let resolver = resolver {
            verifyPhoneNumber(multiFactorSession: resolver.session, hint: hint)
        } else {
            Auth.auth().currentUser?.multiFactor.getSessionWithCompletion({ [weak self] session, error in
                guard let self = self else { return }
                if let error = error {
                    print(error)
                    self.error = error
                    return
                }
                guard let session = session else {
                    //TODO: handle unexpected case
                    return
                }
                self.verifyPhoneNumber(multiFactorSession: session, hint: nil)
            })
        }
    }

    private func verifyPhoneNumber(multiFactorSession: MultiFactorSession, hint: PhoneMultiFactorInfo?) {
        let completion: ((String?, Error?) -> Void) = { [weak self] verificationId, error in
            guard let self = self else { return }

            self.isLoading = false

            if let error = error {
                print(error)
                self.error = error
            }
            guard let verificationId = verificationId else { return }

            self.onResult?(
                .optCodeDidSent(
                    verificationId: verificationId,
                    phoneNumber: self.phoneNumberText,
                    resolver: self.resolver
                )
            )
        }

        if let hint = hint {
            PhoneAuthProvider.provider().verifyPhoneNumber(
                with: hint,
                uiDelegate: nil,
                multiFactorSession: multiFactorSession,
                completion: completion
            )
        } else {
            PhoneAuthProvider.provider().verifyPhoneNumber(
                self.phoneNumberText,
                uiDelegate: nil,
                multiFactorSession: multiFactorSession,
                completion: completion
            )
        }
    }
}
