//
//  VerifyEmailViewModel.swift
//  iOSPOCs
//
//  Created by Roman Sorochak on 23.03.2022.
//

import SwiftUI
import FirebaseAuth
import FirebaseDynamicLinks

// MARK: - VerifyEmailViewModel

class VerifyEmailViewModel: ObservableObject, Resultable, Alertable {

    // MARK: Result

    enum Result {
        case emailVerified
    }

    var onResult: ((Result) -> Void)?

    // MARK: Properties

    @Published var email: String = ""
    @Published var isLoading: Bool = false
    @Published var emailSent: Bool = false
    var error: Error? {
        didSet {
            isErrorAlertShowing = error != nil
        }
    }
    @Published var isErrorAlertShowing: Bool = false

    // MARK: Life cycle

    func onAppear() {
        guard let email = Auth.auth().currentUser?.email else {
            //TODO: handle unexpected scenario
            return
        }
        self.email = email
    }

    func onOpenURL(_ url: URL) {
        handleEmailLinkIfCan(url: url)
    }

    // MARK: Actions

    func sendVerificationEmailAction() {
        sendVerificationEmail()
    }

    // MARK: Private

    private func sendVerificationEmail() {
        isLoading = true

        let actionCodeSettings = ActionCodeSettings()
        actionCodeSettings.url = URL(string: "https://stayopnpoc.page.link")
        actionCodeSettings.handleCodeInApp = true
        actionCodeSettings.setIOSBundleID(Bundle.main.bundleIdentifier!)

        Auth.auth().currentUser?.sendEmailVerification(
            with: actionCodeSettings,
            completion: { [weak self] error in
                guard let self = self else { return }

                self.isLoading = false

                if let error = error {
                    print(error)
                    self.error = error
                    return
                }
                self.emailSent = true
            })
    }

    private func handleEmailLinkIfCan(url: URL) {
        print("url to handle: \(url)")
        DynamicLinks.dynamicLinks().handleUniversalLink(url, completion: { [weak self] dynamicLink, error in
            if let error = error {
                print(error)
                self?.error = error
            }
            guard let link = dynamicLink?.url?.absoluteString else {
                //TODO: handle
                return
            }
            print("emailLink to hanlde: \(link)")

            self?.handle(emailLink: link)
        })
    }

    private func handle(emailLink: String) {
        guard let queryItems = URLComponents(string: emailLink)?.queryItems else {
            return
        }
        print(queryItems)
        guard let code = queryItems.first(where: { $0.name == "oobCode" })?.value else {
            return
        }

        Auth.auth().checkActionCode(code) { [weak self] actionCodeInfo, error in
            guard let self = self else { return }

            if let error = error {
                print(error)
                self.error = error
                return
            }

            print(actionCodeInfo as Any)
            Auth.auth().applyActionCode(code) { [weak self] error in
                guard let self = self else { return }

                if let error = error {
                    print(error)
                    self.error = error
                    return
                }

                self.onResult?(.emailVerified)
            }
        }
    }
}


